@@ -1,5 +1,5 @@
 # Parallel Python Software: http://www.parallelpython.com
-# Copyright (c) 2005-2009, Vitalii Vanovschi
+# Copyright (c) 2005-2010, Vitalii Vanovschi
 # All rights reserved.
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are met:
@@ -23,7 +23,6 @@
 # CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 # ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 # THE POSSIBILITY OF SUCH DAMAGE.
-
 """
 Parallel Python Software, Execution Server
 
@@ -32,7 +31,7 @@ forums
 """
 
 import os
-import thread
+import threading
 import logging
 import inspect
 import sys
@@ -43,12 +42,19 @@ import user
 import cPickle as pickle
 import pptransport
 import ppauto
+import ppcommon
 
-copyright = "Copyright (c) 2005-2009 Vitalii Vanovschi. All rights reserved"
-version = "1.5.7"
+copyright = "Copyright (c) 2005-2010 Vitalii Vanovschi. All rights reserved"
+version = "1.6.0"
 
-# reconnect persistent rworkers in 5 sec
-_RECONNECT_WAIT_TIME = 5
+# Reconnect persistent rworkers in seconds.
+RECONNECT_WAIT_TIME = 5
+
+# Set timeout for socket operations in seconds.
+SOCKET_TIMEOUT = 5
+
+# If set to true prints out the exceptions which are expected.
+SHOW_EXPECTED_EXCEPTIONS = False
 
 # we need to have set even in Python 2.3
 try:
@@ -63,7 +69,6 @@ try:
 except ImportError:
     import popen2
 
-
 class _Task(object):
     """Class describing single task (job)
     """
@@ -71,7 +76,7 @@ class _Task(object):
     def __init__(self, server, tid, callback=None,
             callbackargs=(), group='default'):
         """Initializes the task"""
-        self.lock = thread.allocate_lock()
+        self.lock = threading.Lock()
         self.lock.acquire()
         self.tid = tid
         self.server = server
@@ -93,6 +98,8 @@ class _Task(object):
 
     def __call__(self, raw_result=False):
         """Retrieves result of the task"""
+        if not self.finished and self.server._exiting:
+            raise DestroyedServerError("Server was destroyed before the job completion")
         self.wait()
 
         if not self.unpickled and not raw_result:
@@ -158,8 +165,10 @@ class _Worker(object):
     def stop(self):
         """Stops local worker"""
         self.is_free = False
+        self.t.send('EXIT') # can send any string - it will exit
         self.t.close()
 
+
     def restart(self):
         """Restarts local worker"""
         self.stop()
@@ -177,18 +186,18 @@ class _RWorker(pptransport.CSocketTransport):
     """Remote worker class
     """
 
-    def __init__(self, host, port, secret, message=None, persistent=True):
+    def __init__(self, host, port, secret, server,  message=None, persistent=True):
         """Initializes remote worker"""
+        self.server = server
         self.persistent = persistent
         self.host = host
         self.port = port
         self.secret = secret
         self.address = (host, port)
         self.id = host + ":" + str(port)
-        logging.debug("Creating Rworker id=%s persistent=%s"
+        self.server.logger.debug("Creating Rworker id=%s persistent=%s"
                 % (self.id, persistent))
         self.connect(message)
-        self.is_free = True
 
     def __del__(self):
         """Closes connection with remote server"""
@@ -196,12 +205,12 @@ class _RWorker(pptransport.CSocketTransport):
 
     def connect(self, message=None):
         """Connects to a remote server"""
-        while True:
+        while True and not self.server._exiting:
             try:
-                pptransport.SocketTransport.__init__(self)
-                self._connect(self.host, self.port)
+                pptransport.SocketTransport.__init__(self)                
+                self._connect(self.host, self.port)                
                 if not self.authenticate(self.secret):
-                    logging.error("Authentication failed for host=%s, port=%s"
+                    self.server.logger.error("Authentication failed for host=%s, port=%s"
                             % (self.host, self.port))
                     return False
                 if message:
@@ -209,15 +218,17 @@ class _RWorker(pptransport.CSocketTransport):
                 self.is_free = True
                 return True
             except:
+                if SHOW_EXPECTED_EXCEPTIONS:
+                    self.server.logger.debug("Exception in connect method "
+                            "(possibly expected)", exc_info=True)
                 if not self.persistent:
-                    logging.debug("Deleting from queue Rworker %s"
+                    self.server.logger.debug("Deleting from queue Rworker %s"
                             % (self.id, ))
                     return False
-#                print sys.excepthook(*sys.exc_info())
-                logging.debug("Failed to reconnect with " \
+                self.server.logger.info("Failed to reconnect with " \
                         "(host=%s, port=%i), will try again in %i s"
-                        % (self.host, self.port, _RECONNECT_WAIT_TIME))
-                time.sleep(_RECONNECT_WAIT_TIME)
+                        % (self.host, self.port, RECONNECT_WAIT_TIME))
+                time.sleep(RECONNECT_WAIT_TIME)
 
 
 class _Statistics(object):
@@ -277,8 +288,7 @@ class Server(object):
     default_secret = "epo20pdosl;dksldkmm"
 
     def __init__(self, ncpus="autodetect", ppservers=(), secret=None,
-            loglevel=logging.WARNING, logstream=sys.stderr,
-            restart=False, proto=0):
+            restart=False, proto=2):
         """Creates Server instance
 
            ncpus - the number of worker processes to start on the local
@@ -289,8 +299,6 @@ class Server(object):
            secret - passphrase for network connections, if omitted a default
                    passphrase will be used. It's highly recommended to use a
                    custom passphrase for all network connections.
-           loglevel - logging level
-           logstream - log stream destination
            restart - wheather to restart worker process after each task completion
            proto - protocol number for pickle module
 
@@ -302,25 +310,24 @@ class Server(object):
         if not isinstance(ppservers, tuple):
             raise TypeError("ppservers argument must be a tuple")
 
-        self.__initlog(loglevel, logstream)
-        logging.debug("Creating server instance (pp-" + version+")")
+        self.logger = logging.getLogger('pp')
+        self.logger.info("Creating server instance (pp-" + version+")")
         self.__tid = 0
         self.__active_tasks = 0
-        self.__active_tasks_lock = thread.allocate_lock()
+        self.__active_tasks_lock = threading.Lock()
         self.__queue = []
-        self.__queue_lock = thread.allocate_lock()
+        self.__queue_lock = threading.Lock()
         self.__workers = []
         self.__rworkers = []
         self.__rworkers_reserved = []
-        self.__rworkers_reserved4 = []
         self.__sourcesHM = {}
         self.__sfuncHM = {}
         self.__waittasks = []
-        self.__waittasks_lock = thread.allocate_lock()
-        self.__exiting = False
+        self.__waittasks_lock = threading.Lock()
+        self._exiting = False
         self.__accurate_stats = True
         self.autopp_list = {}
-        self.__active_rworkers_list_lock = thread.allocate_lock()
+        self.__active_rworkers_list_lock = threading.Lock()
         self.__restart_on_free = restart
         self.__pickle_proto = proto
 
@@ -354,13 +361,13 @@ class Server(object):
                 broadcast = host.replace("*", "255")
                 self.auto_ppservers.append(((interface, port),
                         (broadcast, port)))
-        self.__stats_lock = thread.allocate_lock()
+        self.__stats_lock = threading.Lock()
         if secret is not None:
             if not isinstance(secret, types.StringType):
                 raise TypeError("secret must be of a string type")
             self.secret = str(secret)
         elif hasattr(user, "pp_secret"):
-            secret = user["pp_secret"]
+            secret = getattr(user, "pp_secret")
             if not isinstance(secret, types.StringType):
                 raise TypeError("secret must be of a string type")
             self.secret = str(secret)
@@ -368,7 +375,7 @@ class Server(object):
             self.secret = Server.default_secret
         self.__connect()
         self.__creation_time = time.time()
-        logging.info("pp local server started with %d workers"
+        self.logger.info("pp local server started with %d workers"
                 % (self.__ncpus, ))
 
     def submit(self, func, args=(), depfuncs=(), modules=(),
@@ -390,8 +397,8 @@ class Server(object):
         """
 
         # perform some checks for frequent mistakes
-        if self.__exiting:
-            raise RuntimeError("Cannot submit jobs: server"\
+        if self._exiting:
+            raise DestroyedServerError("Cannot submit jobs: server"\
                     " instance has been destroyed")
 
         if not isinstance(args, tuple):
@@ -415,7 +422,7 @@ class Server(object):
         if globals:
             modules += tuple(self.__find_modules("", globals))
             modules = tuple(set(modules))
-            self.__logger.debug("Task %i will autoimport next modules: %s" %
+            self.logger.debug("Task %i will autoimport next modules: %s" %
                     (tid, str(modules)))
             for object1 in globals.values():
                 if isinstance(object1, types.FunctionType) \
@@ -438,7 +445,9 @@ class Server(object):
             # Checks for both classic or new class instances
             if isinstance(arg, types.InstanceType) \
                     or str(type(arg))[:6] == "<class":
-                depfuncs += (arg.__class__, )
+                # do not include source for imported modules
+                if ppcommon.is_not_imported(arg, modules):
+                    depfuncs += tuple(ppcommon.get_class_hierarchy(arg.__class__))
 
         # if there is a function in the arguments add this
         # function to dependancies
@@ -453,7 +462,7 @@ class Server(object):
         self.__queue.append((task, sfunc, sargs))
         self.__queue_lock.release()
 
-        self.__logger.debug("Task %i submited, function='%s'" %
+        self.logger.debug("Task %i submited, function='%s'" %
                 (tid, func.func_name))
         self.__scheduler()
         return task
@@ -522,7 +531,7 @@ class Server(object):
            clusters"""
 
         print "Job execution statistics:"
-        walltime = time.time()-self.__creation_time
+        walltime = time.time() - self.__creation_time
         statistics = self.get_stats().items()
         totaljobs = 0.0
         for ppserver, stat in statistics:
@@ -554,48 +563,43 @@ class Server(object):
         self.__queue.append((task, sfunc, sargs))
         self.__queue_lock.release()
 
-        self.__logger.debug("Task %i inserted" % (task.tid, ))
+        self.logger.debug("Task %i inserted" % (task.tid, ))
         self.__scheduler()
         return task
 
     def connect1(self, host, port, persistent=True):
-        """Conects to a remote ppserver specified by host and port"""
+        """Conects to a remote ppserver specified by host and port"""        
         try:
-            rworker = _RWorker(host, port, self.secret, "STAT", persistent)
+            rworker = _RWorker(host, port, self.secret, self, "STAT", persistent)
             ncpus = int(rworker.receive())
             hostid = host+":"+str(port)
             self.__stats[hostid] = _Statistics(ncpus, rworker)
 
             for x in range(ncpus):
-                rworker = _RWorker(host, port, self.secret, "EXEC", persistent)
+                rworker = _RWorker(host, port, self.secret, self, "EXEC", persistent)
                 self.__update_active_rworkers(rworker.id, 1)
                 # append is atomic - no need to lock self.__rworkers
                 self.__rworkers.append(rworker)
             #creating reserved rworkers
             for x in range(ncpus):
-                rworker = _RWorker(host, port, self.secret, "EXEC", persistent)
+                rworker = _RWorker(host, port, self.secret, self, "EXEC", persistent)
                 self.__update_active_rworkers(rworker.id, 1)
                 self.__rworkers_reserved.append(rworker)
-            #creating reserved4 rworkers
-            for x in range(ncpus*0):
-                rworker = _RWorker(host, port, self.secret, "EXEC", persistent)
-#                    self.__update_active_rworkers(rworker.id, 1)
-                self.__rworkers_reserved4.append(rworker)
-            logging.debug("Connected to ppserver (host=%s, port=%i) \
+            self.logger.debug("Connected to ppserver (host=%s, port=%i) \
                     with %i workers" % (host, port, ncpus))
             self.__scheduler()
         except:
-            pass
-#            sys.excepthook(*sys.exc_info())
+            if SHOW_EXPECTED_EXCEPTIONS:
+                self.logger.debug("Exception in connect1 method (possibly expected)", exc_info=True)
 
     def __connect(self):
         """Connects to all remote ppservers"""
         for ppserver in self.ppservers:
-            thread.start_new_thread(self.connect1, ppserver)
+            ppcommon.start_thread("connect1",  self.connect1, ppserver)
 
-        discover = ppauto.Discover(self, True)
+        self.discover = ppauto.Discover(self, True)
         for ppserver in self.auto_ppservers:
-            thread.start_new_thread(discover.run, ppserver)
+            ppcommon.start_thread("discover.run", self.discover.run, ppserver)
 
     def __detect_ncpus(self):
         """Detects the number of effective CPUs in the system"""
@@ -617,15 +621,6 @@ class Server(object):
         #return the default value
         return 1
 
-    def __initlog(self, loglevel, logstream):
-        """Initializes logging facility"""
-        log_handler = logging.StreamHandler(logstream)
-        log_handler.setLevel(loglevel)
-        LOG_FORMAT = '%(asctime)s %(levelname)s %(message)s'
-        log_handler.setFormatter(logging.Formatter(LOG_FORMAT))
-        self.__logger = logging.getLogger('')
-        self.__logger.addHandler(log_handler)
-        self.__logger.setLevel(loglevel)
 
     def __dumpsfunc(self, funcs, modules):
         """Serializes functions and modules"""
@@ -661,12 +656,12 @@ class Server(object):
                         worker.is_free = False
                         break
                 else:
-                    self.__logger.error("There are no free workers left")
+                    self.logger.error("There are no free workers left")
                     raise RuntimeError("Error: No free workers")
                 self.__add_to_active_tasks(1)
                 try:
                     self.__stats["local"].njobs += 1
-                    thread.start_new_thread(self.__run, task+(worker, ))
+                    ppcommon.start_thread("run_local",  self._run_local, task+(worker, ))
                 except:
                     pass
             else:
@@ -675,7 +670,7 @@ class Server(object):
                         rworker.is_free = False
                         task = self.__queue.pop(0)
                         self.__stats[rworker.id].njobs += 1
-                        thread.start_new_thread(self.__rrun, task+(rworker, ))
+                        ppcommon.start_thread("run_remote",  self._run_remote, task+(rworker, ))
                         break
                 else:
                     if len(self.__queue) > self.__ncpus:
@@ -684,22 +679,10 @@ class Server(object):
                                 rworker.is_free = False
                                 task = self.__queue.pop(0)
                                 self.__stats[rworker.id].njobs += 1
-                                thread.start_new_thread(self.__rrun,
-                                        task+(rworker, ))
+                                ppcommon.start_thread("run_remote",  self._run_remote, task+(rworker, ))                                
                                 break
                         else:
-                            break
-                            # this code will not be executed
-                            # and is left for further releases
-                            if len(self.__queue) > self.__ncpus*0:
-                                for rworker in self.__rworkers_reserved4:
-                                    if rworker.is_free:
-                                        rworker.is_free = False
-                                        task = self.__queue.pop(0)
-                                        self.__stats[rworker.id].njobs += 1
-                                        thread.start_new_thread(self.__rrun,
-                                                task+(rworker, ))
-                                        break
+                                break
                     else:
                         break
 
@@ -716,9 +699,44 @@ class Server(object):
             self.__sourcesHM[hashf] = "".join(sourcelines)
         return self.__sourcesHM[hashf]
 
-    def __rrun(self, job, sfunc, sargs, rworker):
+    def _run_local(self,  job, sfunc, sargs, worker):
+        """Runs a job locally"""
+
+        if self._exiting:
+            return
+        self.logger.info("Task %i started",  job.tid)
+
+        start_time = time.time()
+
+        try:
+            worker.t.csend(sfunc)
+            worker.t.send(sargs)
+            sresult = worker.t.receive()
+        except:
+            if self._exiting:
+                return
+            if SHOW_EXPECTED_EXCEPTIONS:
+                self.logger.debug("Exception in _run_local (possibly expected)", exc_info=True)
+
+        job.finalize(sresult)
+
+        # remove the job from the waiting list
+        if self.__waittasks:
+            self.__waittasks_lock.acquire()
+            self.__waittasks.remove(job)
+            self.__waittasks_lock.release()
+
+        worker.free()
+
+        self.__add_to_active_tasks(-1)
+        if not self._exiting:
+            self.__stat_add_time("local", time.time()-start_time)
+        self.logger.debug("Task %i ended",  job.tid)
+        self.__scheduler()
+
+    def _run_remote(self, job, sfunc, sargs, rworker):
         """Runs a job remotelly"""
-        self.__logger.debug("Task (remote) %i started" % (job.tid, ))
+        self.logger.debug("Task (remote) %i started",  job.tid)
 
         try:
             rworker.csend(sfunc)
@@ -726,8 +744,8 @@ class Server(object):
             sresult = rworker.receive()
             rworker.is_free = True
         except:
-            self.__logger.debug("Task %i failed due to broken network " \
-                    "connection - rescheduling" % (job.tid, ))
+            self.logger.debug("Task %i failed due to broken network " \
+                    "connection - rescheduling",  job.tid)
             self.insert(sfunc, sargs, job)
             self.__scheduler()
             self.__update_active_rworkers(rworker.id, -1)
@@ -744,42 +762,7 @@ class Server(object):
             self.__waittasks.remove(job)
             self.__waittasks_lock.release()
 
-        self.__logger.debug("Task (remote) %i ended" % (job.tid, ))
-        self.__scheduler()
-
-    def __run(self, job, sfunc, sargs, worker):
-        """Runs a job locally"""
-
-        if self.__exiting:
-            return
-        self.__logger.debug("Task %i started" % (job.tid, ))
-
-        start_time = time.time()
-
-        try:
-            worker.t.csend(sfunc)
-            worker.t.send(sargs)
-            sresult = worker.t.receive()
-        except:
-            if self.__exiting:
-                return
-            else:
-                sys.excepthook(*sys.exc_info())
-
-        worker.free()
-
-        job.finalize(sresult)
-
-        # remove the job from the waiting list
-        if self.__waittasks:
-            self.__waittasks_lock.acquire()
-            self.__waittasks.remove(job)
-            self.__waittasks_lock.release()
-
-        self.__add_to_active_tasks(-1)
-        if not self.__exiting:
-            self.__stat_add_time("local", time.time()-start_time)
-        self.__logger.debug("Task %i ended" % (job.tid, ))
+        self.logger.debug("Task (remote) %i ended",  job.tid)
         self.__scheduler()
 
     def __add_to_active_tasks(self, num):
@@ -815,22 +798,29 @@ class Server(object):
         self.__tid += 1
         return self.__tid - 1
 
+    def __del__(self):
+        self._exiting = True
+
     def destroy(self):
         """Kills ppworkers and closes open files"""
-        self.__exiting = True
+        self._exiting = True
         self.__queue_lock.acquire()
         self.__queue = []
         self.__queue_lock.release()
 
         for worker in self.__workers:
-            worker.t.exiting = True
-            if sys.platform.startswith("win"):
-                os.popen('TASKKILL /PID '+str(worker.pid)+' /F')
-            else:
-                try:
+            try:
+                worker.t.close()
+                if sys.platform.startswith("win"):
+                    os.popen('TASKKILL /PID '+str(worker.pid)+' /F')
+                else:
                     os.kill(worker.pid, 9)
                     os.waitpid(worker.pid, 0)
-                except:
-                    pass
+            except:
+                pass
 
+
+class DestroyedServerError(RuntimeError):
+    pass
+    
 # Parallel Python Software: http://www.parallelpython.com
