@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # Parallel Python Software: http://www.parallelpython.com
-# Copyright (c) 2005-2009, Vitalii Vanovschi
+# Copyright (c) 2005-2010, Vitalii Vanovschi
 # All rights reserved.
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are met:
@@ -35,21 +35,23 @@ import logging
 import getopt
 import sys
 import socket
-import thread
+import threading
 import random
 import string
 import time
 import os
 
-import pptransport
+import pp
 import ppauto
-from pp import Server
+import ppcommon
+import pptransport
 
+copyright = "Copyright (c) 2005-2010 Vitalii Vanovschi. All rights reserved"
+version = "1.6.0"
 
-copyright = "Copyright (c) 2005-2009 Vitalii Vanovschi. All rights reserved"
-version = "1.5.7"
+LISTEN_SOCKET_TIMEOUT = 20
 
-# compartibility with Python 2.6
+# compatibility with Python 2.6
 try:
     import hashlib
     sha_new = hashlib.sha1
@@ -58,16 +60,15 @@ except ImportError:
     sha_new = sha.new
 
 
-class _NetworkServer(Server):
+class _NetworkServer(pp.Server):
     """Network Server Class
     """
 
     def __init__(self, ncpus="autodetect", interface="0.0.0.0",
                 broadcast="255.255.255.255", port=None, secret=None,
-                timeout=None, loglevel=logging.WARNING, restart=False,
-                proto=0):
-        Server.__init__(self, ncpus, secret=secret, loglevel=loglevel,
-                restart=restart, proto=proto)
+                timeout=None, restart=False, proto=2):
+        pp.Server.__init__(self, ncpus, secret=secret, restart=restart,
+                proto=proto)
         self.host = interface
         self.bcast = broadcast
         if port is not None:
@@ -77,14 +78,14 @@ class _NetworkServer(Server):
         self.timeout = timeout
         self.ncon = 0
         self.last_con_time = time.time()
-        self.ncon_lock = thread.allocate_lock()
+        self.ncon_lock = threading.Lock()
 
-        logging.debug("Strarting network server interface=%s port=%i"
+        self.logger.debug("Strarting network server interface=%s port=%i"
                 % (self.host, self.port))
         if self.timeout is not None:
-            logging.debug("ppserver will exit in %i seconds if no "\
+            self.logger.debug("ppserver will exit in %i seconds if no "\
                     "connections with clients exist" % (self.timeout))
-            thread.start_new_thread(self.check_timeout, ())
+            ppcommon.start_thread("timeout_check",  self.check_timeout)
 
     def ncon_add(self, val):
         """Keeps track of the number of connections and time of the last one"""
@@ -101,7 +102,7 @@ class _NetworkServer(Server):
                 if idle_time < self.timeout:
                     time.sleep(self.timeout - idle_time)
                 else:
-                    logging.debug("exiting ppserver due to timeout (no client"\
+                    self.logger.debug("exiting ppserver due to timeout (no client"\
                             " connections in last %i sec)", self.timeout)
                     os._exit(0)
             else:
@@ -110,25 +111,35 @@ class _NetworkServer(Server):
     def listen(self):
         """Initiates listenting to incoming connections"""
         try:
-            ssocket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+            self.ssocket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
             # following allows ppserver to restart faster on the same port
-            ssocket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
-            ssocket.bind((self.host, self.port))
-            ssocket.listen(5)
+            self.ssocket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
+            self.ssocket.settimeout(LISTEN_SOCKET_TIMEOUT)
+            self.ssocket.bind((self.host, self.port))
+            self.ssocket.listen(5)
         except socket.error:
-            logging.error("Cannot create socket with port " + str(self.port)
+            self.logger.error("Cannot create socket with port " + str(self.port)
                     + " (port is already in use)")
 
         try:
             while 1:
+                csocket = None
                 #accept connections from outside
-                (csocket, address) = ssocket.accept()
+                try:
+                    (csocket, address) = self.ssocket.accept()
+                except socket.timeout:
+                    pass
+                if self._exiting:
+                    return                
                 #now do something with the clientsocket
                 #in this case, we'll pretend this is a threaded server
-                thread.start_new_thread(self.crun, (csocket, ))
+                if csocket:
+                    ppcommon.start_thread("client_socket",  self.crun, (csocket,  ))
         except:
-            logging.debug("Closing server socket")
-            ssocket.close()
+            if pp.SHOW_EXPECTED_EXCEPTIONS:
+                self.logger.debug("Exception in listen method (possibly expected)", exc_info=True)
+            self.logger.debug("Closing server socket")
+            self.ssocket.close()            
 
     def crun(self, csocket):
         """Authenticates client and handles its jobs"""
@@ -140,9 +151,9 @@ class _NetworkServer(Server):
                 for i in xrange(16)])
         mysocket.send(srandom)
         answer = sha_new(srandom+self.secret).hexdigest()
-        cleintanswer = mysocket.receive()
-        if answer != cleintanswer:
-            logging.warning("Authentification failed, client host=%s, port=%i"
+        clientanswer = mysocket.receive()
+        if answer != clientanswer:
+            self.logger.warning("Authentication failed, client host=%s, port=%i"
                     % csocket.getpeername())
             mysocket.send("FAILED")
             csocket.close()
@@ -151,7 +162,7 @@ class _NetworkServer(Server):
             mysocket.send("OK")
 
         ctype = mysocket.receive()
-        logging.debug("Control message received: " + ctype)
+        self.logger.debug("Control message received: " + ctype)
         self.ncon_add(1)
         try:
             if ctype == "STAT":
@@ -169,18 +180,19 @@ class _NetworkServer(Server):
                     sresult = fun(True)
                     mysocket.send(sresult)
         except:
-            #print sys.excepthook(*sys.exc_info())
-            logging.debug("Closing client socket")
+            if self._exiting:
+                return
+            if pp.SHOW_EXPECTED_EXCEPTIONS:
+                self.logger.debug("Exception in crun method (possibly expected)", exc_info=True)
+            self.logger.debug("Closing client socket")
             csocket.close()
             self.ncon_add(-1)
 
     def broadcast(self):
         """Initiaates auto-discovery mechanism"""
         discover = ppauto.Discover(self)
-        thread.start_new_thread(discover.run,
-                                ((self.host, self.port),
-                                 (self.bcast, self.port)),
-                               )
+        ppcommon.start_thread("server_broadcast",  discover.run,
+                ((self.host, self.port), (self.bcast, self.port)))
 
 
 def parse_config(file_loc):
@@ -191,8 +203,8 @@ def parse_config(file_loc):
     try:
         from configobj import ConfigObj
     except ImportError, ie:
-        print >> sys.stderr, "ERROR: You must have configobj installed to use \
-configuration files. You can still use command line switches."
+        print >> sys.stderr, ("ERROR: You must have config obj installed to use"
+                "configuration files. You can still use command line switches.")
         sys.exit(1)
 
     if not os.access(file_loc, os.F_OK):
@@ -260,13 +272,14 @@ configuration files. You can still use command line switches."
 def print_usage():
     """Prints help"""
     print "Parallel Python Network Server (pp-" + version + ")"
-    print "Usage: ppserver.py [-hdar] [-n proto] [-c config_path]"\
-            " [-i interface] [-b broadcast] [-p port] [-w nworkers]"\
-            " [-s secret] [-t seconds]"
+    print "Usage: ppserver.py [-hdar] [-f format] [-n proto]"\
+            " [-c config_path] [-i interface] [-b broadcast]"\
+            " [-p port] [-w nworkers] [-s secret] [-t seconds]"
     print
     print "Options: "
     print "-h                 : this help message"
-    print "-d                 : debug"
+    print "-d                 : set log level to debug"
+    print "-f format          : log format"
     print "-a                 : enable auto-discovery service"
     print "-r                 : restart worker process after each"\
             " task completion"
@@ -288,17 +301,19 @@ def print_usage():
     print "documentation, examples and support forums"
 
 
-if __name__ == "__main__":
+def create_network_server(argv):
     try:
-        opts, args = getopt.getopt(sys.argv[1:], 
-            "hdarn:c:b:i:p:w:s:t:", ["help"])
+        opts, args = getopt.getopt(argv, "hdarn:c:b:i:p:w:s:t:f:", ["help"])
     except getopt.GetoptError:
         print_usage()
-        sys.exit(1)
+        raise
 
     args = {}
     autodiscovery = False
 
+    log_level = logging.WARNING
+    log_format = "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
+
     for opt, arg in opts:
         if opt in ("-h", "--help"):
             print_usage()
@@ -306,7 +321,10 @@ if __name__ == "__main__":
         elif opt == "-c":
             args, autodiscovery = parse_config(arg)
         elif opt == "-d":
-            args["loglevel"] = logging.DEBUG
+            log_level = logging.DEBUG
+            pp.SHOW_EXPECTED_EXCEPTIONS = True
+        elif opt == "-f":
+            log_format = arg
         elif opt == "-i":
             args["interface"] = arg
         elif opt == "-s":
@@ -326,12 +344,23 @@ if __name__ == "__main__":
         elif opt == "-t":
             args["timeout"] = int(arg)
 
+    log_handler = logging.StreamHandler()
+    log_handler.setFormatter(logging.Formatter(log_format))
+    logging.getLogger("pp").setLevel(log_level)
+    logging.getLogger("pp").addHandler(log_handler)
+
     server = _NetworkServer(**args)
     if autodiscovery:
         server.broadcast()
+    return server    
+    
+
+if __name__ == "__main__":
+    server = create_network_server(sys.argv[1:])
     server.listen()
-    #have to destroy it here explicitelly otherwise an exception
+    #have to destroy it here explicitly otherwise an exception
     #comes out in Python 2.4
     del server
+    
 
 # Parallel Python Software: http://www.parallelpython.com
