@@ -1,5 +1,5 @@
 # Parallel Python Software: http://www.parallelpython.com
-# Copyright (c) 2005-2009, Vitalii Vanovschi
+# Copyright (c) 2005-2010, Vitalii Vanovschi
 # All rights reserved.
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are met:
@@ -34,8 +34,10 @@ import struct
 import socket
 import logging
 
-copyright = "Copyright (c) 2005-2009 Vitalii Vanovschi. All rights reserved"
-version = "1.5.7"
+copyright = "Copyright (c) 2005-2010 Vitalii Vanovschi. All rights reserved"
+version = "1.6.0"
+
+TRANSPORT_SOCKET_TIMEOUT = 60
 
 # compartibility with Python 2.6
 try:
@@ -145,6 +147,7 @@ class SocketTransport(Transport):
             self.socket = socket1
         else:
             self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        self.socket.settimeout(TRANSPORT_SOCKET_TIMEOUT)
         self.scache = {}
 
     def send(self, data):
@@ -200,5 +203,5 @@ class CPipeTransport(PipeTransport, CTransport):
 
 class CSocketTransport(SocketTransport, CTransport):
     pass
-
+    
 # Parallel Python Software: http://www.parallelpython.com
