@@ -1,5 +1,5 @@
 # Parallel Python Software: http://www.parallelpython.com
-# Copyright (c) 2005-2009, Vitalii Vanovschi
+# Copyright (c) 2005-2010, Vitalii Vanovschi
 # All rights reserved.
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are met:
@@ -35,8 +35,8 @@ import StringIO
 import cPickle as pickle
 import pptransport
 
-copyright = "Copyright (c) 2005-2009 Vitalii Vanovschi. All rights reserved"
-version = "1.5.7"
+copyright = "Copyright (c) 2005-2010 Vitalii Vanovschi. All rights reserved"
+version = "1.6.0"
 
 
 def import_module(name):
@@ -52,7 +52,7 @@ def preprocess(msg):
     fobjs = [compile(fsource, '<string>', 'exec') for fsource in fsources]
     for module in imports:
         try:
-            globals()[module.split('.')[0]] = __import__(module)
+            globals()[module.split('.')[0]] = import_module(module)
         except:
             print "An error has occured during the module import"
             sys.excepthook(*sys.exc_info())
@@ -83,7 +83,7 @@ class _WorkerProcess(object):
 
                 for __fobj in __fobjs:
                     try:
-                        eval(__fobj)
+                        exec __fobj
                         globals().update(locals())
                     except:
                         print "An error has occured during the " + \
@@ -91,7 +91,7 @@ class _WorkerProcess(object):
                         sys.excepthook(*sys.exc_info())
 
                 __args = pickle.loads(__sargs)
-
+            
                 __f = locals()[__fname]
                 try:
                     __result = __f(*__args)
@@ -106,7 +106,7 @@ class _WorkerProcess(object):
                 self.t.send(__sresult)
                 self.sout.truncate(0)
         except:
-            print "Fatal error has occured during the function execution"
+            print "A fatal error has occured during the function execution"
             sys.excepthook(*sys.exc_info())
             __result = None
             __sresult = pickle.dumps((__result, self.sout.getvalue()),
